#######################################
# Unified and final version of REGEX
"""
🔹 Key Improvements & Selection Criteria
IP_REGEX (IPv4 & CIDR): CSAP's version was better as it avoids false positives in domain names.
Domain Matching: CSAP regex handled spaces and special characters better.
Email Detection: Used the more comprehensive version from CSAP.
URL_REGEX: Merged both to ensure URLs with and without schemes are handled.
Hashes (MD5, SHA1, SHA256, etc.): CTIX had better negation rules, preventing accidental extractions from larger text blobs.
Windows Registry Keys & File Paths: Kept the more precise CSAP version, which handles multiple OS path formats.
MAC Addresses: CTIX version was slightly better optimized for strict format checking.
"""
##########################################
IP_REGEX = r"(?i)(?<![a-zA-Z0-9:.])(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]?[0-9])(?:\.(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]?[0-9])){3}(?![a-zA-Z0-9/.])"

DOMAIN_REGEX = r"(?i)(?:^|\s|['\",])((?:(?:[A-Z0-9_](?:[A-Z0-9_-]{0,61}[A-Z0-9_])?\.)+)(?:[A-Z0-9-]{2,63}(?<!-)))(?!(?:/[^\s/]+))(?![a-zA-Z0-9@])"

EMAIL_REGEX = (
    r"(?<!@)\b[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]{2,}\b(?!@)"  # noqa
)

FIND_EMAIL_REGEX = r"[a-z\u00a1-\uffffA-Z0-9_.+-]+@[a-z\u00a1-\uffffA-Z0-9-]+\.[a-zA-Z0-9-.]+"

CVE_REGEX = r"CVE-\d{4}-\d{4,7}"

URL_REGEX = r"""
(?<![a-z0-9])((?:https?|ftps?):\/\/(?:[^\s,]+(?::\S*)?@)?(?:(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}|\[[0-9a-f:\.]+\]|[a-z0-9_](?:[a-z0-9_-]{0,61}[a-z0-9_])?(?:\.(?!-)[a-z0-9_-]{1,63}(?!-))*\.(?!-)(?:[a-z_-]{2,63}|xn--[a-z0-9]{1,59})(?!-)\.?|localhost)(?::\d{2,5})?(?:[/?#][^\"'\s,]*)?)(?![a-z0-9])
|
(?<![a-z0-9])
(
    # Match URLs with optional scheme
    (?:https?:\/\/)?
    (?:[^\s,]+(?::\S*)?@)?
    (?:
        [a-z0-9_](?:[a-z0-9_-]{0,61}[a-z0-9_])?
        (?:\.(?!-)[a-z0-9_-]{1,63}(?!-))*
        \.(?!-)(?:[a-z_-]{2,63}|xn--[a-z0-9]{1,59})(?!-)\.?
        | localhost
    )
    (?::\d{2,5})?
    (?:\/[^\s,]+)
    (?![a-z0-9])
)
"""

SHA256_REGEX = r"(?i)(?<![a-zA-Z0-9])[a-fA-F0-9]{64}(?![a-zA-Z0-9])"

MD5_REGEX = r"(?i)(?<![^\s\"',])[a-fA-F0-9]{32}(?![^\s\"',])"

SHA1_REGEX = r"(?i)(?<![a-zA-Z0-9])[a-fA-F0-9]{40}(?![a-zA-Z0-9])"

UUID_REGEX = r"\b[0-9a-f]{8}\b-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-\b[0-9a-f]{12}\b"

IPv4_CIDR = r"(?i)(?<![a-zA-Z.0-9])(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}/(?:3[0-2]|[12][0-9]|[1-9])(?![a-zA-Z.0-9])"

IPv6_REGEX = r"""(?:^\s|,|)((?:[0-9a-fA-F]{1,4}:){7}(?:[0-9a-fA-F]{1,4})|(?:[0-9a-fA-F]{1,4}:){1,7}:|:(?::[0-9a-fA-F]{1,4}){1,7}|(?:[0-9a-fA-F]{1,4}:){1,3}(?::[0-9a-fA-F]{1,4}){1,4}|(?:[0-9a-fA-F]{1,4}:){1,4}(?::[0-9a-fA-F]{1,4}){1,3}|(?:[0-9a-fA-F]{1,4}:){1,2}(?::[0-9a-fA-F]{1,4}){1,5}|(?:[0-9a-fA-F]{1,4}:){1,5}(?::[0-9a-fA-F]{1,4}){1,2}|(?:[0-9a-fA-F]{1,4}:){1}(?::[0-9a-fA-F]{1,4}){1,6}|(?:[0-9a-fA-F]{1,4}:){1,6}(?::[0-9a-fA-F]{1,4}){1}|::)(?=\s|,|\r|\'|\"|$)"""  # noqa

SHA224_REGEX = r"(?i)(?<![a-zA-Z0-9])[a-fA-F0-9]{56}(?![a-zA-Z0-9])"

SHA384_REGEX = r"(?i)(?<![a-zA-Z0-9])[a-fA-F0-9]{96}(?![a-zA-Z0-9])"

SHA512_REGEX = r"(?i)(?<![a-zA-Z0-9])[a-fA-F0-9]{128}(?![a-zA-Z0-9])"

SSDEEP_REGEX = r"(?i)(?<![a-zA-Z0-9/+:.])[0-9]+:[a-zA-Z0-9/+]{3,64}:[a-zA-Z0-9/+]{3,64}(?![a-zA-Z0-9/+:.])"

DIRECTORY_REGEX = r"""(?:^|\s|,|'|")((?:[a-zA-Z0-9-_+=!@#$%^&*\.\(\)\{\}\[\]\;]+:[\\\/]|\\\\[^\\\/]+\\|\/)(?:[^\\\/:*?'"<>|\r\n,]+[\\\/])*(?![^\\\/:*?'"<>|\r\n,]+\.[a-zA-Z0-9]+$)(?:(?:[^\\\/:*?'"<>|\r\n,]+[\\\/])|(?:[^\\\/:*?'"<>|\r\n,.]+)))(?=,|'|"|\r|\n|$)"""

FILE_PATH_REGEX = r"""(?:^|\s|,|'|")((?:[a-zA-Z0-9-_+=!@#$%^&*\.\(\)\{\}\[\]\;]+:[\\\/]|\\\\[^\\\/]+\\|\/)(?:[^\\\/:*?'"<>|\r\n,]+[\\\/])*[^\\\/:*?"<>|\r\n,]*\.\w+)(?=\s|,|'|"|$)"""

AUTONOMOUS_SYSTEM_REGEX = r"(?i)\b(AS\d{1,10})\b"

WINDOWS_REGISTRY_KEY = r"(?:HKEY_LOCAL_MACHINE|HKLM|HKEY_CURRENT_USER|HKCU|HKEY_CLASSES_ROOT|HKCR|HKEY_USERS|HKU|HKEY_CURRENT_CONFIG|HKCC)\\(?:[a-zA-Z0-9\s_@\-\^!#.\:\/\$%&+={}\[\]\\*]+\\)*[a-zA-Z0-9\s_@\-\^!#.\:\/\$%&+={}\[\]\\*]+\b"

MAC_ADDRESS_REGEX = r"(?i)(?<![a-zA-Z0-9:])([a-fA-F0-9]{2}(?::[a-fA-F0-9]{2}){5})(?![a-zA-Z0-9:])"

VALID_HOST_REGEX = "(?!-)[A-Z\d-]{1,63}(?<!-)$"

WHIRLPOOL_REGEX = r"^[a-fA-F0-9]{128}$"

SHA3512_REGEX = r"^[a-fA-F0-9]{128}$"

SHA3384_REGEX = "^[a-fA-F0-9]{96}$"

SHA3256_REGEX = r"^[a-fA-F0-9]{64}$"

SHA3224_REGEX = r"^[a-fA-F0-9]{56}$"
